/* eslint-disable quotes */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import { accountNavsType } from '../../utils/propTypes';
import styles from './AccountPanel.scss';
import getStyleName from '../../utils/getStyleName';
import UserDefault from '../../assets/user-default.svg';
// import UserLogin from '../../assets/user-login.svg';
import MoreIcon from './icon_more.svg';
import AnueIcon from './icon_anue_rainbow.svg';
import UserDefaultAvatar from '../../assets/user-avatar.svg';
import Config from './config/navs.json';

const getNavConfig = (navCofig, cnyesBaseUrl) => {
  const nav = navCofig || Config.items;
  const replaceLink = str => str.replace('{cnyesBaseUrl}', cnyesBaseUrl);

  nav.forEach((item, i) => {
    // defaultUrl
    nav[i].defaultUrl = replaceLink(item.defaultUrl);
    // notify
    if (item.notify) {
      item.notify.forEach((n, j) => {
        const gotoUrl = nav[i].notify[j].gotoUrl;

        nav[i].notify[j].gotoUrl = replaceLink(gotoUrl);
      });
    }
  });

  return nav;
};

export default class AccountPanel extends PureComponent {
  static propTypes = {
    isOpen: PropTypes.bool,
    isLogin: PropTypes.bool,
    profile: PropTypes.shape({
      identity_id: PropTypes.string,
      name: PropTypes.string,
      nickname: PropTypes.string,
      avatar: PropTypes.string,
    }),
    login: PropTypes.func,
    logout: PropTypes.func,
    highlights: PropTypes.arrayOf(PropTypes.string),
    onClosePanel: PropTypes.func,
    onClickNav: PropTypes.func,
    cnyesBaseUrl: PropTypes.string.isRequired,
    navs: accountNavsType,
  };

  static defaultProps = {
    isOpen: false,
    isLogin: false,
    profile: {
      identity_id: '',
      name: '',
      avatar: '',
    },
    onClosePanel: () => {},
    onClickNav: () => {},
    login: () => {},
    logout: () => {},
    highlights: [],
    cnyesBaseUrl: 'cnyes.com',
    navs: undefined,
  };

  constructor(props) {
    super(props);
    this.navConfig = getNavConfig(props.navs, props.cnyesBaseUrl);
    this.memberInfoLink = `https://www.${props.cnyesBaseUrl}/member/info`;
    this.state = {
      isOpenState: false,
      avatar: UserDefaultAvatar,
    };
  }

  componentWillReceiveProps(nextProps) {
    const { isOpen } = nextProps;
    const { isOpenState } = this.state;

    if (isOpen !== isOpenState) {
      this.setState({ isOpenState: isOpen });

      this.image = new Image();
      this.image.addEventListener('load', this.avatarLoaded);
      this.image.src = this.props.profile.avatar;
    }
  }

  componentDidUpdate(prevProps, prevState) {
    const { isOpenState } = this.state;

    if (prevState.isOpenState !== isOpenState) {
      if (window) {
        if (isOpenState) {
          // move to next tic to prevent trigger `handleDocClick` when click avatar
          setTimeout(() => {
            window.addEventListener('click', this.handleDocClick);
          }, 0);
        } else {
          window.removeEventListener('click', this.handleDocClick);
        }
      }
    }
  }

  onClickClose = () => {
    this.setState({ isOpenState: false });
    this.props.onClosePanel();

    if (this.image) {
      this.image.removeEventListener('load', this.avatarLoaded);
    }
  };

  onClickNav = () => {
    this.setState({ isOpenState: false });
    this.props.onClickNav();
  };

  handleDocClick = e => {
    if (!this.accountPanelRef.contains(e.target) && this.state.isOpenState) {
      this.onClickClose();
    }
  };

  avatarLoaded = () => {
    this.setState({ avatar: this.props.profile.avatar });
  };

  renderMemberInfo() {
    const { isLogin, profile, login } = this.props;

    if (isLogin) {
      const userAvatar = this.state.avatar || UserDefaultAvatar;
      const userName = profile.nickname || profile.name;

      return (
        <a href={this.memberInfoLink} className={styles.panel__member} data-tid="account-panel-member">
          <img className={styles.panel__member__avatar} src={userAvatar} alt="" />
          <div className={cx(styles.panel__member__info, styles['panel__member__info--login'])}>
            <div>{userName}</div>
            <span>查看個人資訊</span>
          </div>
          <img className={styles.panel__member__more} alt="前往查看個人資訊" src={MoreIcon} />
        </a>
      );
    }

    return (
      <div className={styles.panel__member}>
        <img className={styles.panel__member__avatar} src={UserDefault} alt="登入會員" />
        <div className={styles.panel__member__info}>
          <div>Hello, 鉅亨會員</div>
        </div>
        <button className={styles.panel__member__login} onClick={login} data-tid="account-panel-login">
          登入
        </button>
      </div>
    );
  }

  render() {
    const { isLogin, logout, highlights } = this.props;
    const { isOpenState } = this.state;

    return (
      <aside
        className={cx(styles.panel__wrapper, 'theme-gradient', {
          [getStyleName(styles, 'panel__wrapper--active')]: isOpenState,
        })}
        ref={ref => {
          this.accountPanelRef = ref;
        }}
      >
        <header>
          <div className={styles.header__title}>
            <img className={styles.header__icon} alt="Anue 會員中心" src={AnueIcon} />
            <span className={styles.header__txt}>會員中心</span>
          </div>
          <button className={styles.header__close} onClick={this.onClickClose} data-tid="account-panel-close" />
        </header>
        <main>
          {this.renderMemberInfo()}
          <nav className={styles.panel__nav}>
            {this.navConfig.map(item => {
              let isHighlight = false;
              let link = item.defaultUrl;

              if (item.notify) {
                const notify = item.notify;
                const filteredNotify = notify.filter(n => highlights.indexOf(n.id) > -1);

                if (filteredNotify.length > 0) {
                  isHighlight = true;
                  link = filteredNotify[0].gotoUrl;
                }
              }

              return (
                <a
                  className={cx(styles.panel__link, {
                    [getStyleName(styles, 'panel__link--highlight')]: isHighlight,
                  })}
                  key={item.id}
                  href={link}
                  onClick={this.onClickNav}
                  data-tid={`account-panel-${item.id}`}
                >
                  {item.title}
                  <img className={styles.panel__link__more} alt={`前往${item.title}`} src={MoreIcon} />
                </a>
              );
            })}
            <div className={styles.nav__line} />
            {isLogin && (
              <button className={styles.nav__logout} onClick={logout} data-tid="account-panel-logout">
                登出帳號
              </button>
            )}
          </nav>
        </main>
      </aside>
    );
  }
}
