/* eslint-disable react/no-array-index-key */
/* eslint-disable no-unused-expressions */

/**
 * https://www.cnblogs.com/slongs/p/10189439.html
 */
import React, { useRef, useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import { DELAY_TIME, DOT_POSITION, RIGHT, LEFT } from '../constants';
import ArrowLeft from '../../../assets/icon-slider-arrow-left.svg';
import ArrowRight from '../../../assets/icon-slider-arrow-right.svg';
import styles from '../Carousel.scss';

const cx = classNames.bind(styles);

/**
 * @param {number} [width = 300] - 顯示寬度
 * @param {number} [height = 193] - 顯示高度
 * @param {boolean} [hasArrow = false] - 是否需要顯示左右箭頭
 * @param {boolean} [autoPlay = true] - 是否自動播放
 * @param {number} [selected = 0] - 指定從第幾個 Slide 開始播
 * @param {number}} [speed = 10000] - 切換速度(ms)，預設是 10 秒
 * @param {string}} [dotPosition = fixed] - 分頁按鈕對齊方式，共有二種 float | fixed
 * @param {function}} 完成切換的 Callback 處理
 */

const Carousel = ({ width, height, hasArrow, autoPlay, selected, speed, dotPosition, onChange, children }) => {
  const timerRef = useRef();
  const [currentCarousel, setCurrentCarousel] = useState(0);
  const [animationStep, setAnimationStep] = useState(0);
  const [isShowArrow, setIsShowArrow] = useState(false);

  // 取得目前輪播物件個數
  const childrenCount = React.Children.count(children) || 0;

  /**
   * @description 更改目前循環播放的下一個座標
   * @param {string} direction 輪播方向 (left | right)
   */
  const handlerCarousel = (direction = RIGHT) => {
    const directionValue = direction === LEFT ? -1 : 1;

    // 座標不為 -1 或是目前不是最後一個的情況下，進行遞增或遞減
    if (currentCarousel % (childrenCount + 1) !== childrenCount && currentCarousel >= 0) {
      setAnimationStep(selected);
      // 當座標在邊界時，會取消動畫過場的秒數，當目前座標不是邊界時，要恢復動畫過場秒數，不然無法正確播放動畫
      setCurrentCarousel(prevState => {
        const current = prevState + directionValue;

        return current % (childrenCount + 1);
      });
    }
  };

  /**
   * @description 停止輪播
   */
  const stopCarousel = () => {
    timerRef.current && clearInterval(timerRef.current);
  };

  /**
   * @description 開始輪播
   */
  const startCarousel = () => {
    stopCarousel();

    timerRef.current = setInterval(() => {
      handlerCarousel();
    }, speed);
  };

  /**
   * @description 顯示/隱藏箭頭
   */
  const handleMouseEnter = () => {
    hasArrow && setIsShowArrow(true);
  };

  const handleMouseLeave = () => {
    hasArrow && setIsShowArrow(false);
  };

  /**
   * @description 監聽動畫結束
   */
  const handleTransitionEnd = () => {
    // 當輪播最後一個時，取消過場動畫，並將座標位置重設為 0
    if (currentCarousel % (childrenCount + 1) === childrenCount) {
      setAnimationStep(0);
      setCurrentCarousel(0);
      // 當輪播第一個時，取消過場動畫，並將座標位置設為最後一個
    } else if (currentCarousel < 0) {
      setAnimationStep(0);
      setCurrentCarousel(childrenCount - 1);
    }
  };

  /**
   * @description 設定點擊箭頭後，切換顯示對應方向的 Slide 內容
   *
   * @param {string} direction - 移動方向 (left | right)
   */
  const handleArrowClick = direction => {
    if (autoPlay) {
      startCarousel();
    }

    handlerCarousel(direction);
  };

  /**
   * @description 判斷導航小點點的 active 狀態
   */
  const getIndicatorsActive = index => {
    return (
      currentCarousel === index ||
      currentCarousel === index + childrenCount ||
      (currentCarousel < 0 && index === childrenCount - 1)
    );
  };

  /**
   * @description 點擊指定輪播項目切換
   */
  const onGoToCarousel = index => {
    setCurrentCarousel(index);
  };

  /**
   * @description 開始執行
   */
  useEffect(
    () => {
      setAnimationStep(selected);
      autoPlay && startCarousel();

      if (!autoPlay) {
        stopCarousel();
      } else {
        startCarousel();
      }

      return () => {
        stopCarousel();
      };
    },
    [autoPlay]
  );

  useEffect(
    () => {
      if (onChange) {
        const index = currentCarousel === childrenCount || currentCarousel === -1 ? 0 : currentCarousel;

        onChange(index);
      }
    },
    [currentCarousel]
  );

  const totalCount = childrenCount + 2;
  const containerWidth = width.toString().includes('%') ? width : `${width}px`;
  const containerHeight = height.toString().includes('%') ? height : `${height}px`;
  const carouselStyle = { width: containerWidth, height: containerHeight };
  const transitionStyle = {
    transition: `transform ${animationStep}s`,
    width: `${totalCount * 100}%`,
    height: containerHeight,
    transform: `translateX(${-100 / totalCount * (currentCarousel + 1)}%)`,
  };
  const firstChild = children ? children[0] : null;
  const lastChild = children ? children[childrenCount - 1] : null;

  /**
   * @description 繪制箭頭
   *
   * @param {string} direction - 移動方向 (left | right)
   */
  const renderCarouselArrow = (direction = LEFT) => {
    const arrowIcon = direction === LEFT ? ArrowLeft : ArrowRight;

    return (
      <div
        className={cx('carousel__arrows__arrow', { 'carousel__arrows__arrow--active': isShowArrow })}
        onClick={() => handleArrowClick(direction)}
      >
        <img src={arrowIcon} alt={`${direction} arrow`} />
      </div>
    );
  };

  /**
   * @description 繪製輪播項目
   *
   * @param {ReactNode} child - 輪播項目
   * @param {number} index - 索引值
   */
  const renderCarouselItem = (child, index = 0) => {
    if (!child) return null;

    const carouselItemStyle = {
      width: `${100 / totalCount}%`,
    };

    return (
      <div className={cx('carousel__body__item')} style={carouselItemStyle} key={`carousel_item_${index}`}>
        {child}
      </div>
    );
  };

  const renderIndicators = () => {
    if (!children) return null;

    return (
      <div className={cx('footer', { float: dotPosition === DOT_POSITION.FLOAT })}>
        <ul className={cx('footer__indicators')}>
          {children.map((_, index) => {
            const isActive = getIndicatorsActive(index);

            return (
              <li
                key={`carousel_indicator_${index}`}
                onClick={() => onGoToCarousel(index)}
                className={cx('footer__indicators__indicator', { 'footer__indicators__indicator--active': isActive })}
              />
            );
          })}
        </ul>
      </div>
    );
  };

  return (
    <div className={cx('carousel-container')} style={carouselStyle}>
      <div className={cx('carousel')} onMouseEnter={handleMouseEnter} onMouseLeave={handleMouseLeave}>
        <div className={cx('carousel__body')} onTransitionEnd={handleTransitionEnd} style={transitionStyle}>
          {renderCarouselItem(lastChild, 0)}
          {children && children.map((item, index) => renderCarouselItem(item, index + 1))}
          {renderCarouselItem(firstChild, totalCount)}
        </div>
        <div className={cx('carousel__arrows')}>
          {hasArrow && renderCarouselArrow(LEFT)}
          {hasArrow && renderCarouselArrow(RIGHT)}
        </div>
      </div>
      {renderIndicators()}
    </div>
  );
};

Carousel.propTypes = {
  width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  height: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
  hasArrow: PropTypes.bool,
  autoPlay: PropTypes.bool,
  children: PropTypes.ReactNode.isRequired,
  selected: PropTypes.number,
  speed: PropTypes.number,
  dotPosition: PropTypes.string,
  onChange: PropTypes.func,
};

Carousel.defaultProps = {
  width: 300,
  height: 193,
  hasArrow: true,
  autoPlay: true,
  selected: 1,
  speed: DELAY_TIME,
  dotPosition: DOT_POSITION.FLOAT,
  onChange: null,
};

export default Carousel;
