/* eslint-disable no-nested-ternary */
/* eslint-disable no-unused-expressions */

/**
 * 在 iOS 6 當中，Safari 加入了 Smart App Banner 的支援
 * 在 App 專屬頁面的最上面出現了一個 Banner，有 App 的名字、圖示以及評價。如果是在 iPad 的 Safari 開啟，甚至還會看到 App 的截圖。
 * 在 Banner 的右方，有個連結可以開啟 App Store 的連結，如果在使用者的 device 當中已經安裝了這個 App，那麼會有個可以開啟 App 的連結。
 * 要加入 Smart App Banners 到你的網頁相當容易，只要在 head 區域當中加入 meta tag：
 * ```
 * <meta name="apple-itunes-app" content="app-id=542383077"/>
 * ```
 * 其中，app-id 的部份就是你 App 所屬的 ID。
 * 當使用者已經安裝了你的 App 的時候，Smart App Banners 會直接讓使用者開啟你的 App
 *
 * ## react-smartbanner:
 * https://github.com/patw0929/react-smartbanner
 *
 * https://github.com/patw0929/react-smartbanner/blob/master/src/components/SmartBanner.js
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import Cookie from 'universal-cookie';
import gaDataset from '../../utils/gaDataset';
import AppIcon from '../../assets/icon-anue-app.png';
import styles from './SmartBanner.scss';

const OS_TYPE = {
  IOS: 'iOS',
  ANDROID: 'Android',
};

const isClient = typeof window !== 'undefined';
const cx = classNames.bind(styles);
const appConfig = {
  ios: {
    appMeta: 'apple-itunes-app',
    storeLink: 'https://apps.apple.com/tw/app/id',
    // iconRels: ['apple-touch-icon-precomposed', 'apple-touch-icon'],
  },
  android: {
    appMeta: 'google-play-app',
    storeLink: 'http://play.google.com/store/apps/details?id=',
    // iconRels: [
    //   'android-touch-icon',
    //   'apple-touch-icon-precomposed',
    //   'apple-touch-icon',
    // ],
  },
};

let ua;
let cookie;

const oneDayLater = new Date(Date.now() + 1000 * 60 * 60 * 24);

class SmartBanner extends PureComponent {
  static propTypes = {
    onShow: PropTypes.func,
    onClose: PropTypes.func,
    gaEvent: PropTypes.shape({
      dataPrefix: PropTypes.arrayOf(PropTypes.string),
      category: PropTypes.string,
      action: PropTypes.string,
      label: PropTypes.string,
    }),
  };

  static defaultProps = {
    onShow: () => {},
    onClose: () => {},
    gaEvent: {
      dataPrefix: ['global'],
      category: '下載NewsApp',
      action: 'Click',
      label: '',
    },
  };

  constructor(props) {
    super(props);

    if (isClient) {
      ua = require('ua-parser-js'); // eslint-disable-line global-require
    }

    cookie = new Cookie();
    this.isCloseBanner = !!cookie.get('smart-banner-closed') || false;

    this.state = {
      type: '',
      appId: '',
      settings: {},
      closed: false,
    };
  }

  componentWillMount() {
    if (!this.isCloseBanner) {
      this.setType();
    }
  }

  setSettingsByType() {
    this.setState(
      prevState => ({ settings: appConfig[prevState.type] }),
      () => {
        this.state.type && this.parseAppId();
      }
    );
  }

  setType() {
    if (isClient) {
      const agent = ua(window.navigator.userAgent);
      let type;

      // iOS >= 6 has native support for Smart Banner
      // if (agent.os.name === OS_TYPE.IOS && agent.browser.name !== 'Mobile Safari') {
      if (agent.os.name === OS_TYPE.IOS) {
        type = 'ios';
      } else if (agent.os.name === OS_TYPE.ANDROID) {
        type = 'android';
      }

      type &&
        this.setState({ type }, () => {
          this.setSettingsByType();
        });
    }
  }

  parseAppId() {
    if (!isClient) {
      return '';
    }

    const meta = window.document.querySelector(`meta[name="${this.state.settings.appMeta}"]`);

    if (!meta) {
      return '';
    }

    let appId = '';

    if (this.state.type === 'windows') {
      appId = meta.getAttribute('content');
    } else {
      const content = /app-id=([^\s,]+)/.exec(meta.getAttribute('content'));

      appId = content && content[1] ? content[1] : appId;
    }

    this.setState({ appId }, () => {
      if (this.props.onShow && typeof this.props.onShow === 'function') {
        this.props.onShow(true);
      }
    });

    return appId;
  }

  closeHandler = () => {
    this.setState({ closed: true }, () => {
      if (this.props.onClose && typeof this.props.onClose === 'function') {
        this.props.onClose(true);

        // 關閉 Smart App Banner 的顯示，下次出現是一天後
        if (cookie) {
          cookie.set('smart-banner-closed', 1, { path: '/', expires: oneDayLater });
        }
      }
    });
  };

  downloadHandler = () => {
    const { appId, settings } = this.state;
    const link = `${settings.storeLink}${appId}`;

    isClient && window.open(link);
    this.closeHandler();
  };

  render() {
    if (!isClient) {
      return null;
    }

    // Don't show banner when:
    // 1) if device isn't iOS or Android
    // 2) website is loaded in app,
    // 3) user dismissed banner,
    // 4) or we have no app id in meta
    if (!this.state.type || window.navigator.standalone || this.isCloseBanner || !this.state.appId) {
      return null;
    }

    const { gaEvent } = this.props;
    const { appId, type, closed } = this.state;

    const gaDataSet = gaDataset({
      dataPrefix: gaEvent.dataPrefix.map(prefix => `data-${prefix}-ga`),
      category: gaEvent.category || '下載NewsApp',
      action: gaEvent.action || 'Click',
      label: gaEvent.label && gaEvent.label !== '' ? gaEvent.label : type === OS_TYPE.IOS ? 'App Store' : 'Google Play',
    });

    return (
      <div className={cx('smart-banner', { hide: closed })}>
        <div className={cx('smart-banner-container')}>
          <button type="button" className={cx('smart-banner-close')} aria-label="close" onClick={this.closeHandler}>
            &times;
          </button>
          <img src={AppIcon} className={cx('smart-banner-icon')} alt="鉅亨網 APP" />
          <p>
            精選財經深度觀點盡在 <strong>鉅亨App</strong>
          </p>
        </div>
        {appId && (
          <button
            type="button"
            className={cx('smart-banner-download')}
            aria-label="download"
            onClick={this.downloadHandler}
            {...gaDataSet}
          >
            下載
          </button>
        )}
      </div>
    );
  }
}

export default SmartBanner;
